**********************************
*   ORCHESTRA-90CC DSKCON PATCH    
**********************************

DSKCON		EQU		$C004
DSKCONVAR	EQU		$C006
DEFDRV		EQU		$095A

NMIVEC		EQU		$0109
IRQVEC		EQU		$010C

; DRIVEPAK
USD0		EQU		$014D

; COCOSDC
SDC0		EQU		$014E

; VARIABLES

STACK		EQU		$380

DSKVAR		EQU		$3F9
MAXDRV		EQU		$3FB
DRVNUM		EQU		$3FC
DRVPAK		EQU		$3FD
COCOSDC     EQU		$3FE
VALUE		EQU		$3FF

; ORCH90 CODE
PLAYCMD		EQU		$D85B

********************************
* INIT
********************************
 
			ORG		$3F68

LOADER		CLR		<$71
			ORCC	#$50
			LDX		#$8000

LOAD02		LDA		,X
			STA		$FFDF			; DISABLE ROM
			STA		,X+
			STA		$FFDE			; ENABLE ROM
			CMPX	#$C000
			BNE		LOAD02
			STA		$FFDF			; DISABLE ROM

			LDD		#$E000			; COPY PROGRAM TO ROM SPACE
			STD		<$41
			LDD		#$4000
			STD		<$47
			LDD		#$6000
			STD		<$43
			JSR		$AC20

INIT		CLR		MAXDRV
			CLR		DRVNUM
			CLR		DRVPAK
			CLR		COCOSDC

			CLR		$FFDE		; ENABLE ROM

			JSR		SAVEVARS	; SAVE DISK VARIABLES

			LDD		DSKCONVAR
			STD		DSKVAR

			LDA		DEFDRV		; GET CURRENT DRIVE NUMBER
			STA		DRVNUM

			LDD		$D75F		; DETECT RGB/HDB-DOS
			CMPA	#$7E
			BNE		ISSDC
			CMPB	#$DA
			BNE		ISDPAK
			LDA		#255		; 255 drives
			BRA		INIT02

ISDPAK		CMPB	#$DF
			BNE		ISSDC
			LDA		#1
			STA		DRVPAK
			CLR		DRVNUM
			LDA		#255		; 255 drives
			BRA		INIT02

ISSDC		LDD		$DFFE
			CMPD	#$0203
			BNE		FLOPPY
			LDA		#1
			STA		COCOSDC
			CLR		DRVNUM
			LDA		#255		; 255 drives
			BRA		INIT02

FLOPPY		LDA		#3			; 4 drives
			
INIT02		STA		MAXDRV		; SET MAX DRV

			CLR		$FFDF		; DISABLE ROM

			; MOVE DISK IO CODE to $03C0

			LDX		#DISKIO
			LDY		#$3C0

INIT03		LDA		,X+
			STA		,Y+
			CMPX	#DISKIOEND
			BNE		INIT03

			JMP		$C000		; JUMP TO CART


********************************
* DSKCON
********************************

			ORG		$53AB

DISK		PSHS	U,Y,X,B

			LDU		DSKVAR

			LDA		<$9F		; OPC - 00-READ, 20-WRITE
			LSRA
			LSRA
			LSRA
			LSRA
			LSRA
			ADDA	#2
			STA		,U			; SET OPC - 2-READ, 3-WRITE
			
			LDA		DRVNUM
			TST		DRVPAK		; IS DRIVEPAK?
			BEQ		DISK02
			STA		USD0
			CLRA
			BRA		DISK03

DISK02		TST		COCOSDC		; IS COCOSDC?
			BEQ		DISK03
			STA		SDC0
			CLRA

DISK03		STA		1,U			; SET DRIVE NUMBER
			LDA		<$A3		; TRACK
			STA		2,U
			LDA		<$A4		; SECTOR
			STA		3,U
			LDX		<$7E		; load sector buffer
			STX		4,U

			JSR		$03C0		; DISKIO
        
			LDA		6,U			; GET STATUS
			STA		<$A5		; STORE STATUS

			PULS	U,Y,X,B,PC

********************************
* GETNUM
********************************

			ORG		$540D		; skip routine at $D408-$D40C

GETNUM		CLR		VALUE

			JSR		$C908		; clear screen
			JSR		$D70D		; skip space

READ		LDA		,U			; read character
			BMI		RETURN		; branch if end of string
			CMPA	#$70		; 0
			BLO		ERROR		; not a digit
			CMPA	#$79		; 9
			BHI		ERROR		; not a digit
			LDA		VALUE
			LDB		#10
			MUL
			TSTA
			BNE		ERROR		; value higher than 255
			STB		VALUE
			LDA		,U+
			SUBA	#$70		; convert char to decimal
			ADDA	VALUE
			BCS		ERROR		; value higher than 255
			STA		VALUE
			BRA		READ

RETURN		LDA		VALUE
			CMPA	MAXDRV		; validate drive number
			BHI		ERROR		; value higher than maximum
			STA		DRVNUM		; set drive
			RTS

ERROR		JMP		$D206

********************************
* SAVEVARS
********************************

SAVEVARS	PSHS	X,U
			LDX		#$97E
			LDU		#$380

SAVEV02		LDA		,X+
			STA		,U+
			CMPX	#$98B
			BNE		SAVEV02

			PULS	X,U,PC

********************************
* RESTVARS
********************************

RESTVARS	PSHS	X,U
			LDX		#$97E
			LDU		#$380

RESTV02		LDA		,U+
			STA		,X+
			CMPX	#$98B
			BNE		RESTV02
			
			PULS	X,U,PC

********************************
* PLAY
********************************

PLAY		JSR		SAVEVARS+$8000	; save disk variables
			JMP		PLAYCMD			; jump to original command

********************************
* PLAYED
********************************

PLAYED		JSR		RESTVARS+$8000	; restore disk variables
			JSR		$CEB7			; call original subroutine
			RTS

********************************
* DISK IO
********************************

DISKIO		CLR		$FFDE		; ENABLE ROM
			JSR		[DSKCON] 	; CALL DSKCON
			CLR		$FFDF		; DISABLE ROM

			RTS

DISKIOEND

********************************
* DIR COMMAND PATCH
********************************

			ORG		$5337

			JSR		GETNUM+$8000

********************************
* GAT RELOCATION PATCH
********************************

			ORG		$539E
			LDX		#$0700

			ORG		$5326
			LDX		#$0800

			ORG		$5375
			CMPX	#$0800

********************************
* NMI HANDLER PATCH
********************************

			ORG		$5F2F
			LDX		#$D8AF

			ORG		$5F37
			LDX		#$D8A1

********************************
* DISK CRASH PATCH
********************************

			ORG		$4EC1
			LDD		#$343C

			ORG		$4EC8
			NOP

********************************
* PLAY COMMAND PATCH
********************************

			ORG		$4B83
			FDB		PLAY+$8000

********************************
* AFTER PLAY COMMAND PATCH
********************************

			ORG		$5D8F

			JSR		PLAYED+$8000

********************************
* DISABLE XFER COMMAND
********************************

			ORG		$4B94
			FCC		$FE

********************************
* RELOCATE STACK
********************************

			ORG		$4000
			LDS		#STACK

			ORG		$4BE9
			LDS		#STACK

			ORG		$4EB3
			LDS		#STACK-2

			ORG		$525D
			LDS		#STACK

			END		LOADER